//
//  VerticalCell.swift
//  CollectionCrash
//
//  Created by Miika Fabritius on 29/07/2018.
//  Copyright © 2018 Miika Fabritius. All rights reserved.
//

import UIKit

final class VerticalCell: UIView {
    
    var verticalTitle: String? {
        didSet {
            guard oldValue != verticalTitle else { return }
            setNeedsDisplay()
        }
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        backgroundColor = UIColor(white: 0.1, alpha: 1.0)
    }
    
    override func draw(_ rect: CGRect) {
        guard let text = verticalTitle else { return }
        let context = UIGraphicsGetCurrentContext()!
        context.saveGState()
        context.translateBy(x: -0, y: rect.height)
        context.rotate(by: -90 * CGFloat.pi/180)
        let textRect = CGRect(x: 0, y: 0, width: rect.height, height: rect.width)
        let textTextContent = text
        let textStyle = NSMutableParagraphStyle()
        textStyle.alignment = .center
        let textFontAttributes = [.font: UIFont.systemFont(ofSize: UIFont.labelFontSize), .foregroundColor: UIColor(white: 0.7, alpha: 1.0), .paragraphStyle: textStyle, ] as [NSAttributedStringKey: Any]
        
        let textTextHeight: CGFloat = textTextContent.boundingRect(with: CGSize(width: textRect.width, height: CGFloat.infinity),
                                                                   options: .usesLineFragmentOrigin, attributes: textFontAttributes, context: nil).height
        context.saveGState()
        context.clip(to: textRect)
        textTextContent.draw(in: CGRect(x: textRect.minX, y: textRect.minY + (textRect.height - textTextHeight) / 2, width: textRect.width, height: textTextHeight), withAttributes: textFontAttributes)
        context.restoreGState()
        
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}
