//
//  ViewController.swift
//  CollectionCrash
//
//  Created by Miika Fabritius on 29/07/2018.
//  Copyright © 2018 Miika Fabritius. All rights reserved.
//

import UIKit
import CollectionKit


final class PresetBrowser: UIView {
    
    struct Collection {
        let title: String
        let itemCount: Int
    }
    
    private let collectionView = CollectionView()
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        backgroundColor = .darkGray
        addSubview(collectionView)
        
        var collections = [Collection]()
        collections.append(Collection(title: "Section 1", itemCount: 5))
        collections.append(Collection(title: "Section 2", itemCount: 5))
        collections.append(Collection(title: "Section 3", itemCount: 5))
        collections.append(Collection(title: "Section 4", itemCount: 5))
        collections.append(Collection(title: "Section 5", itemCount: 5))
        collections.append(Collection(title: "Section 6", itemCount: 5))
        
        let sections: [Provider] = collections.map { collection in
            return BasicProvider(
                identifier: collection.title,
                dataSource: ArrayDataSource(data: Array(1...collection.itemCount)),
                viewSource: ClosureViewSource(viewUpdater: { (view: UILabel, data: Int, index: Int) in
                    view.backgroundColor = UIColor.white
                    view.text = "\(data)"
                    view.font = UIFont.systemFont(ofSize: 32, weight: .bold)
                    view.textAlignment = .center
                }),
                sizeSource:  { (index, data, maxSize) -> CGSize in return CGSize(width: maxSize.height-20, height: maxSize.height-20) },
                layout: FlowLayout(spacing: 5, justifyContent: .center).transposed().inset(by: UIEdgeInsetsMake(0, 10, 0, 10))
            )
        }
        let provider = ComposedHeaderProvider(
            headerViewSource: ClosureViewSource(
                viewUpdater: { (view: VerticalCell, data, index) in view.verticalTitle = data.section.identifier
            }),
            headerSizeSource: { (index, data, maxSize) -> CGSize in return CGSize(width: 50, height: maxSize.height) },
            sections: sections
        )
        
        provider.tapHandler = { context in
            print(context)
        }
        
        provider.layout = StickyLayout(rootLayout: FlowLayout()).transposed()
        collectionView.provider = provider
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        collectionView.frame = bounds
    }
    
    required init?(coder aDecoder: NSCoder) { fatalError("init(coder:) has not been implemented") }
}

class ViewController: UIViewController {
    
    let presetBrowser = PresetBrowser(frame: CGRect.zero)

    override func viewDidLoad() {
        super.viewDidLoad()
        view.addSubview(presetBrowser)
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        presetBrowser.frame = CGRect(x: 0, y: view.frame.height-200, width: view.frame.width, height: 200)
    }
}

